/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef AP_TRIGGERS_HH
#define AP_TRIGGERS_HH

#include "ap-channels.hh"
#include "DatEnv.hh"
#include "OperStateCondList.hh"
#include "TSeries.hh"
#include <vector>
#include <string>
#include <iosfwd>

namespace xsil {
   class MetaIO;
}

class triggers {
public:

   //-----------------------------------   Trigger selection structure.
   struct trig_select {
      void critter(std::ostream& out) const;
      std::string field;
      std::string string;
      double      number;
      enum  {
	 s_pass,
	 s_string,
	 s_num_ge,
	 s_num_lt
      } type;
   };

   //-----------------------------------  Trigger veto structure
   struct trig_veto {
      std::string channel;
      double      val_min;
      double      val_max;
   };

   typedef Time::ulong_t gps_t;
   typedef std::vector<trig_select> _select_list;
   typedef std::vector<trig_veto>   _veto_list;

public:
   triggers(void);
   ~triggers(void);
   void add_select(const trig_select& sel);
   void add_veto(const trig_veto& vet);
   double average(void) const;
   bool no_retrigger(void) const;
   void read_triggers(const std::vector<std::string>& file,
		      gps_t start, gps_t stop, Interval dt);
   bool select_trigger(const xsil::MetaIO& mio) const;
   const _select_list& select_list(void) const;
   std::string select_string(void) const;
    const TSeries& series(void) const;
   void set_debug(int level);
   void set_no_retrigger(bool no_retrig);
   void set_series(const Time& t0, Interval dt, size_t nbin);
   void set_table_name(const std::string& tab);
   void set_time_field(const std::string& name);
   void set_time_ns_field(const std::string& name);
   const std::string& table_name(void) const;
   const std::string& time_field(void) const;
   const std::string& time_ns_field(void) const;
   size_t total(void) const;

   /**  Veto triggers based on channel values.
    */
   void veto_stats(void);
   
   /**  Veto triggers based on channel values.
    */
   void veto_triggers(const std::string& name, const TSeries& value);

   /**  Reference the trigger veto list.
    */
   const _veto_list& veto_list(void) const;

private:
   //-------------------------------------  Trigger file and selection
   std::vector<std::string> _trig_path;
   std::string _table_name;
   std::string _time_field;
   std::string _time_ns_field;
   bool        _no_retrigger;
   int         _debug;

   //-------------------------------------  Trigger data
   _select_list _trig_select_list;
   _veto_list  _trig_veto_list;
   TSeries     _series;
   size_t      _total;
   size_t      _nVetoed;
   size_t      _nEnt;
};

inline double
triggers::average(void) const {
   size_t nBin = _series.getNSample();
   if (!nBin) return 0.0;
   return double(_total) / double(nBin);
}

inline bool
triggers::no_retrigger(void) const {
   return _no_retrigger;
}
inline const triggers::_select_list&
triggers::select_list(void) const {
   return _trig_select_list;
}

inline const TSeries&
triggers::series(void) const {
   return _series;
}

inline const std::string& 
triggers::table_name(void) const {
   return _table_name;
}

inline const std::string& 
triggers::time_field(void) const {
   return _time_field;
}

inline const std::string& 
triggers::time_ns_field(void) const {
   return _time_ns_field;
};

inline size_t
triggers::total(void) const {
   return _total;
}

inline const triggers::_veto_list&
triggers::veto_list(void) const {
   return _trig_veto_list;
}

#endif
